import "dart:convert";

import "package:crypto/crypto.dart" as crypto;
import 'package:dart_style/dart_style.dart';
import "package:matchable_builder/matchable_builder.dart";

class InterceptableCache extends Cache {
  final String path;
  final String className;
  final String generatedClassName;
  final String code;
  final bool isAbstract;

  String get fileName => Uri.file(path).pathSegments.last;

  String get generatedFileName => "${fileName}.${filePathHash}.dart";

  String get filePathHash => crypto.md5.convert(Utf8Encoder().convert(path)).toString();

  String get formattedCode => DartFormatter().format(code);

  InterceptableCache({
    this.path,
    this.className,
    this.generatedClassName,
    this.code,
    this.isAbstract,
  });

  Map<String, dynamic> toJson() {
    return {
      "path": path,
      "className": className,
      "generatedClassName": generatedClassName,
      "code": code,
      "isAbstract": isAbstract,
    };
  }

  static InterceptableCache fromJson(Map json) {
    return InterceptableCache(
      path: json["path"],
      className: json["className"],
      generatedClassName: json["generatedClassName"],
      code: json["code"],
      isAbstract: json["isAbstract"],
    );
  }
}
