import "dart:async";

import "package:build/build.dart";
import "package:dart_style/dart_style.dart";
import "package:interceptable/src/annotation/interceptable.dart";
import "package:interceptable/src/builder/interceptable_cache.dart";
import "package:interceptable/src/builder/util/template_utils.dart";
import "package:matchable_builder/matchable_builder.dart" as mb;
import "package:renderable/renderable.dart";

class InterceptableCollector extends mb.MatchableBuilder {
  static final DartFormatter formatter = DartFormatter();

  InterceptableCollector(BuilderOptions options) : super(options);

  @override
  Map<String, List<String>> get buildExtensions => {
        ".dart": [".interceptable.json"],
      };

  @override
  mb.Matcher get matcher => mb.Matcher.and([
        mb.ElementAnnotationMatcher<Intercept>(),
        mb.FileSchemaMatcher("package"),
      ]);

  @override
  FutureOr<void> generate(List<mb.Element> elements, BuildStep buildStep) async {
    if (elements.isEmpty) {
      return null;
    }
    String filePath = buildStep.inputId.uri.toString();
//    String filePathHash = crypto.md5.convert(Utf8Encoder().convert(filePath)).toString();
    var libraryElement = AnalyzerUtils.parseLibraryElement(elements.first.library);

    // import superclass
    libraryElement.imports.add(ImportElement(uri: "${buildStep.inputId.uri}"));

    libraryElement.topLevelElements.whereType<ClassElement>().forEach((classElement) {
      String classOriginName = classElement.name;

      classElement.name = "\$${classOriginName}Intercept";

      // create subclass
      classElement.supertype = InterfaceType(
        name: classOriginName,
        typeParameters: classElement.typeParameters,
      );

      // remove fields
      classElement.fields.clear();
      // enable all accessor
      classElement.accessors
        ..forEach((accessor) => ElementUtils.changeSynthetic(accessor, false))
        // generate accessor template
        ..forEach((accessor) => accessor.statements = [
              RawStatement(TemplateUtils.stringFromAccessor(classOriginName, accessor)),
            ])
        ..forEach((accessor) => accessor.isAbstract = false);
      // generate method template
      classElement.methods
        ..forEach((method) => method.statements = [
              RawStatement(TemplateUtils.stringFromMethod(classOriginName, method)),
            ])
        ..forEach((method) => method.isAbstract = false);
    });

    var cache = InterceptableCache(
      path: filePath,
      className: elements.first.name,
      generatedClassName: "\$${elements.first.name}Intercept",
      code: libraryElement.render(),
      isAbstract: (elements.first as mb.ClassElement).isAbstract,
    );

    mb.CacheHelper cacheHelper = mb.CacheHelper(buildStep);
    cacheHelper.writeAssetToExtension(".interceptable.json", cache);
  }
}
