import "dart:async";
import "dart:io";

import "package:build/build.dart";
import "package:build/src/builder/build_step.dart";
import "package:build/src/builder/builder.dart";
import 'package:dart_style/dart_style.dart';
import "package:interceptable/src/builder/interceptable_cache.dart";
import "package:matchable_builder/matchable_builder.dart";
import "package:renderable/renderable.dart" as r;

class InterceptableWriter extends MatchableCombiningBuilder {
  InterceptableWriter(BuilderOptions options) : super(options);

  @override
  Map<String, List<String>> get buildExtensions => {
        "interceptable.locator": ["interceptable.dart"],
      };

  @override
  Map<String, CacheResolver> get resolveCaches => {
        "**/*.interceptable.json": InterceptableCache.fromJson,
      };

  @override
  FutureOr<void> generate(Map<String, List<Object>> resolvedCaches, BuildStep buildStep) {
    String folderPath = Uri.file(buildStep.inputId.path).resolve(".").toString();
    String entryPath = Uri.file(buildStep.inputId.path).resolve("./interceptable.dart").toString();
    final List<InterceptableCache> caches = List<InterceptableCache>.from(
      resolvedCaches["**/*.interceptable.json"],
    ).where((e) => !e.isAbstract).toList();
    if (caches.isEmpty) {
      return null;
    }
    List<r.ImportElement> imports = [
      r.ImportElement(uri: "package:interceptable/interceptable.dart"),
    ];
    for (var cache in caches) {
      File generateFile = File("$folderPath${cache.fileName}.${cache.filePathHash}.dart");
      // write file only when content changes
      if (!generateFile.existsSync() || generateFile.readAsStringSync() != cache.code) {
        generateFile.writeAsStringSync(cache.formattedCode);
      }
      imports.add(r.ImportElement(uri: cache.path));
      imports.add(r.ImportElement(uri: cache.generatedFileName));
    }

    var libraryElement = r.LibraryElement(
      imports: imports,
      topLevelElements: [
        r.FunctionElement(
          name: "initializeInterceptable",
//          returnType: r.DartType.void_,
          statements: [
            r.RawStatement("Interceptable.interceptableMapping.load({"),
            r.RawStatement(caches
                .map((cache) => "${cache.className}: () => ${cache.generatedClassName}(),")
                .join("\n")),
            r.RawStatement("});"),
          ],
        ),
      ],
    );

    final outputId = AssetId(buildStep.inputId.package, entryPath);
    var formatted = DartFormatter().format(libraryElement.render());
    buildStep.writeAsString(outputId, formatted);
  }
}
