import "package:interceptable/src/runtime/contract/filter.dart";
import "package:interceptable/src/runtime/filter_chain.dart";
import "package:interceptable/src/runtime/getter_target.dart";
import "package:interceptable/src/runtime/interceptable_mapping.dart";
import "package:interceptable/src/runtime/method_target.dart";
import "package:interceptable/src/runtime/setter_target.dart";
import "package:reflect_metadata/reflect_metadata.dart";

class Interceptable {
  static String _METADATA_KEY = "interceptable|interceptor";

  static InterceptableMapping interceptableMapping = InterceptableMapping();

  static void addInterceptor(
    Type type,
    String declarationName,
    Filter interceptor,
  ) {
    if (ReflectMetadata.hasMetadata(_METADATA_KEY, type, declarationName)) {
      (ReflectMetadata.getMetadata(_METADATA_KEY, type, declarationName) as List).add(interceptor);
      return;
    }

    ReflectMetadata.defineMetadata(
      _METADATA_KEY,
      [interceptor],
      type,
      declarationName,
    );
  }

  static newInstance(Type type) {
    return interceptableMapping.get(type);
  }

  static Object applyGetterInterceptor(
    Object instance,
    Type type,
    String declarationName,
    Object value,
  ) {
    var interceptors = ReflectMetadata.getMetadata(_METADATA_KEY, type, declarationName);

    if (interceptors == null) {
      return value;
    }

    var filterChain = FilterChain(interceptors);

    return filterChain.execute(
      GetterTarget(instance, type, declarationName, value),
    );
  }

  static Object applySetterInterceptor(
    Object instance,
    Type type,
    String declarationName,
    Object value,
  ) {
    var interceptors = ReflectMetadata.getMetadata(_METADATA_KEY, type, declarationName);

    if (interceptors == null) {
      return value;
    }

    var filterChain = FilterChain(interceptors);

    return filterChain.execute(
      SetterTarget(instance, type, declarationName, value),
    );
  }

  static Object applyMethodInterceptor(
    Object instance,
    Type type,
    String declarationName,
    dynamic method,
  ) {
    var interceptors = ReflectMetadata.getMetadata(_METADATA_KEY, type, declarationName);

    if (interceptors == null) {
      return Function.apply(method, []);
    }

    var filterChain = FilterChain(interceptors);

    return filterChain.execute(
      MethodTarget(instance, type, declarationName, method),
    );
  }
}
